@@ -33,8 +33,9 @@
 #include <thrust/detail/vector_base.h>
 #include <thrust/device_allocator.h>
 
-#include <initializer_list>
-#include <utility>
+#include <cuda/std/initializer_list>
+#include <cuda/std/utility>
+
 #include <vector>
 
 THRUST_NAMESPACE_BEGIN
@@ -96,6 +97,20 @@ public:
       : Parent(n)
   {}
 
+  //! This constructor creates a \p device_vector with the given size, performing only default-initialization instead of
+  //! value-initialization.
+  //! \param n The number of elements to initially create.
+  device_vector(size_type n, default_init_t)
+      : Parent(n, default_init_t{})
+  {}
+
+  //! This constructor creates a \p device_vector with the given size, without initializing elements. It mandates that
+  //! the element type is trivially default-constructible.
+  //! \param n The number of elements to initially create.
+  device_vector(size_type n, no_init_t)
+      : Parent(n, no_init_t{})
+  {}
+
   /*! This constructor creates a \p device_vector with the given
    *  size.
    *  \param n The number of elements to initially create.
@@ -143,7 +158,7 @@ public:
    *  \param v The device_vector to move.
    */
   device_vector(device_vector&& v)
-      : Parent(std::move(v))
+      : Parent(::cuda::std::move(v))
   {}
 
   /*! Move constructor moves from another \p device_vector.
@@ -151,7 +166,7 @@ public:
    *  \param alloc The allocator to use by this device_vector.
    */
   device_vector(device_vector&& v, const Alloc& alloc)
-      : Parent(std::move(v), alloc)
+      : Parent(::cuda::std::move(v), alloc)
   {}
 
   /*! Copy assign operator copies another \p device_vector with the same type.
@@ -168,7 +183,7 @@ public:
    */
   device_vector& operator=(device_vector&& v)
   {
-    Parent::operator=(std::move(v));
+    Parent::operator=(::cuda::std::move(v));
     return *this;
   }
 
@@ -231,7 +246,7 @@ public:
   /*! This constructor builds a \p device_vector from an intializer_list.
    *  \param il The intializer_list.
    */
-  device_vector(std::initializer_list<T> il)
+  device_vector(::cuda::std::initializer_list<T> il)
       : Parent(il)
   {}
 
@@ -239,14 +254,14 @@ public:
    *  \param il The intializer_list.
    *  \param alloc The allocator to use by this device_vector.
    */
-  device_vector(std::initializer_list<T> il, const Alloc& alloc)
+  device_vector(::cuda::std::initializer_list<T> il, const Alloc& alloc)
       : Parent(il, alloc)
   {}
 
   /*! Assign an \p intializer_list with a matching element type
    *  \param il The intializer_list.
    */
-  device_vector& operator=(std::initializer_list<T> il)
+  device_vector& operator=(::cuda::std::initializer_list<T> il)
   {
     Parent::operator=(il);
     return *this;
@@ -295,6 +310,18 @@ public:
      */
     void resize(size_type new_size, const value_type &x = value_type());
 
+    //! \brief Resizes this vector to the specified number of elements, performing default-initialization instead of
+    //!         value-initialization.
+    //! \param new_size Number of elements this vector should contain.
+    //! \throw std::length_error If n exceeds max_size().
+    void resize(size_type new_size, default_init_t);
+
+    //! \brief Resizes this vector_base to the specified number of elements, without initializing elements. It mandates
+    //! that the element type is trivially default-constructible.
+    //! \param new_size Number of elements this vector_base should contain.
+    //! \throw std::length_error If n exceeds max_size().
+    void resize(size_type new_size, no_init_t);
+
     /*! Returns the number of elements in this vector.
      */
     size_type size() const;
