@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import operator
 import warnings
 
@@ -5,6 +7,10 @@ import numpy
 
 import cupy
 from cupy import _core
+from cupy._core import _accelerator
+from cupy.cuda import cub
+from cupy.cuda import common
+from cupy.cuda import runtime
 
 
 # rename builtin range for use in functions that take a range argument
@@ -94,11 +100,11 @@ def _get_outer_edges(a, range):
                 'supplied range of [{}, {}] is not finite'.format(
                     first_edge, last_edge))
     elif a.size == 0:
-        first_edge = 0.0
-        last_edge = 1.0
+        first_edge = 0
+        last_edge = 1
     else:
-        first_edge = float(a.min())
-        last_edge = float(a.max())
+        first_edge = a.min()
+        last_edge = a.max()
         if not (cupy.isfinite(first_edge) and cupy.isfinite(last_edge)):
             raise ValueError(
                 'autodetected range of [{}, {}] is not finite'.format(
@@ -171,7 +177,7 @@ def _get_bin_edges(a, bins, range):
     return bin_edges
 
 
-def histogram(x, bins=10, range=None, weights=None, density=False):
+def histogram(x, bins=10, range=None, density=False, weights=None):
     """Computes the histogram of a set of data.
 
     Args:
@@ -216,10 +222,43 @@ def histogram(x, bins=10, range=None, weights=None, density=False):
 
     if weights is None:
         y = cupy.zeros(bin_edges.size - 1, dtype=cupy.int64)
-        # TODO(leofang): we temporarily remove CUB histogram support for now,
-        # see cupy/cupy#7698. When it's ready, revert the commit that checked
-        # in this comment to restore the support.
-        _histogram_kernel(x, bin_edges, bin_edges.size, y)
+        for accelerator in _accelerator.get_routine_accelerators():
+            # CUB uses int for bin counts
+            # TODO(leofang): support >= 2^31 elements in x?
+            if (accelerator == _accelerator.ACCELERATOR_CUB
+                    and x.size <= 0x7fffffff and bin_edges.size <= 0x7fffffff):
+                # Need to ensure the dtype of bin_edges as it's needed for both
+                # the CUB call and the correction later
+                assert isinstance(bin_edges, cupy.ndarray)
+                if numpy.issubdtype(x.dtype, numpy.integer):
+                    bin_type = float
+                else:
+                    bin_type = numpy.result_type(bin_edges.dtype, x.dtype)
+                    if (bin_type == numpy.float16 and
+                            not common._is_fp16_supported()):
+                        bin_type = numpy.float32
+                    x = x.astype(bin_type, copy=False)
+                acc_bin_edge = bin_edges.astype(bin_type, copy=True)
+                # CUB's upper bin boundary is exclusive for all bins, including
+                # the last bin, so we must shift it to comply with NumPy
+                if x.dtype.kind in 'ui':
+                    acc_bin_edge[-1] += 1
+                elif x.dtype.kind == 'f':
+                    last = acc_bin_edge[-1]
+                    acc_bin_edge[-1] = cupy.nextafter(last, last + 1)
+                if runtime.is_hip:
+                    y = y.astype(cupy.uint64, copy=False)
+                out = cub.cub_histogram(x, y, acc_bin_edge)
+                if out is None:
+                    # fallback to CuPy impl
+                    continue
+                else:
+                    y = out
+                if runtime.is_hip:
+                    y = y.astype(cupy.int64, copy=False)
+                break
+        else:
+            _histogram_kernel(x, bin_edges, bin_edges.size, y)
     else:
         simple_weights = (
             cupy.can_cast(weights.dtype, cupy.float64) or
@@ -519,10 +558,21 @@ def bincount(x, weights=None, minlength=None):
 
     if weights is None:
         b = cupy.zeros((size,), dtype=numpy.intp)
-        # TODO(leofang): we temporarily remove CUB histogram support for now,
-        # see cupy/cupy#7698. When it's ready, revert the commit that checked
-        # in this comment to restore the support.
-        _bincount_kernel(x, b)
+
+        for accelerator in _accelerator.get_routine_accelerators():
+            # CUB uses int for bin counts
+            # TODO(leofang): support >= 2^31 elements in x?
+            if (not runtime.is_hip
+                    and accelerator == _accelerator.ACCELERATOR_CUB
+                    and x.size <= 0x7fffffff and size <= 0x7fffffff):
+                out = cub.cub_histogram(x, b, size+1)
+                if out is None:
+                    continue
+                else:
+                    b = out
+                    break
+        else:
+            _bincount_kernel(x, b)
     else:
         b = cupy.zeros((size,), dtype=numpy.float64)
         _bincount_with_weight_kernel(x, weights, b)
