@@ -24,6 +24,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
+from __future__ import annotations
+
 
 import cupy
 from cupy._core._scalar import get_typename
@@ -71,7 +73,12 @@ _sawtooth_kernel = cupy.ElementwiseKernel(
         out = nan("0xfff8000000000000ULL");
     }
 
-    const T tmod { fmod( t, 2.0 * M_PI ) };
+    // Use Python-compatible modulo: result is always in [0, 2*pi)
+    // C fmod can return negative values for negative inputs
+    T tmod { fmod( t, 2.0 * M_PI ) };
+    if ( tmod < 0 ) {
+        tmod += 2.0 * M_PI;
+    }
     const bool mask2 { ( ( 1 - mask1 ) && ( tmod < ( w * 2.0 * M_PI ) ) ) };
 
     if ( mask2 ) {
@@ -85,7 +92,6 @@ _sawtooth_kernel = cupy.ElementwiseKernel(
     y = out;
     """,
     "_sawtooth_kernel",
-    options=("-std=c++11",),
 )
 
 
@@ -140,7 +146,12 @@ _square_kernel = cupy.ElementwiseKernel(
         y = nan("0xfff8000000000000ULL");
     }
 
-    const T tmod { fmod( t, 2.0 * M_PI ) };
+    // Use Python-compatible modulo: result is always in [0, 2*pi)
+    // C fmod can return negative values for negative inputs
+    T tmod { fmod( t, 2.0 * M_PI ) };
+    if ( tmod < 0 ) {
+        tmod += 2.0 * M_PI;
+    }
     const bool mask2 { ( ( 1 - mask1 ) && ( tmod < ( w * 2.0 * M_PI ) ) ) };
 
     if ( mask2 ) {
@@ -154,7 +165,6 @@ _square_kernel = cupy.ElementwiseKernel(
 
     """,
     "_square_kernel",
-    options=("-std=c++11",),
 )
 
 
@@ -220,7 +230,6 @@ _gausspulse_kernel_F_F = cupy.ElementwiseKernel(
     yI = yenv * cos( 2 * M_PI * fc * t);
     """,
     "_gausspulse_kernel",
-    options=("-std=c++11",),
 )
 
 _gausspulse_kernel_F_T = cupy.ElementwiseKernel(
@@ -231,7 +240,6 @@ _gausspulse_kernel_F_T = cupy.ElementwiseKernel(
     yI = yenv * cos( 2 * M_PI * fc * t);
     """,
     "_gausspulse_kernel",
-    options=("-std=c++11",),
 )
 
 _gausspulse_kernel_T_F = cupy.ElementwiseKernel(
@@ -247,7 +255,6 @@ _gausspulse_kernel_T_F = cupy.ElementwiseKernel(
     yQ = yenv * l_yQ;
     """,
     "_gausspulse_kernel",
-    options=("-std=c++11",),
 )
 
 _gausspulse_kernel_T_T = cupy.ElementwiseKernel(
@@ -263,7 +270,6 @@ _gausspulse_kernel_T_T = cupy.ElementwiseKernel(
     yQ = yenv * l_yQ;
     """,
     "_gausspulse_kernel",
-    options=("-std=c++11",),
 )
 
 
@@ -379,7 +385,6 @@ _chirp_phase_lin_kernel_real = cupy.ElementwiseKernel(
     phase = cos(temp + phi);
     """,
     "_chirp_phase_lin_kernel",
-    options=("-std=c++11",),
 )
 
 _chirp_phase_lin_kernel_cplx = cupy.ElementwiseKernel(
@@ -392,7 +397,6 @@ _chirp_phase_lin_kernel_cplx = cupy.ElementwiseKernel(
     phase = Y(cos(temp + phi), cos(temp + phi + M_PI/2) * -1);
     """,
     "_chirp_phase_lin_kernel",
-    options=("-std=c++11",),
 )
 
 _chirp_phase_quad_kernel = cupy.ElementwiseKernel(
@@ -412,7 +416,6 @@ _chirp_phase_quad_kernel = cupy.ElementwiseKernel(
     phase = cos(temp + phi);
     """,
     "_chirp_phase_quad_kernel",
-    options=("-std=c++11",),
 )
 
 _chirp_phase_log_kernel = cupy.ElementwiseKernel(
@@ -430,7 +433,6 @@ _chirp_phase_log_kernel = cupy.ElementwiseKernel(
     phase = cos(temp + phi);
     """,
     "_chirp_phase_log_kernel",
-    options=("-std=c++11",),
 )
 
 _chirp_phase_hyp_kernel = cupy.ElementwiseKernel(
@@ -448,7 +450,6 @@ _chirp_phase_hyp_kernel = cupy.ElementwiseKernel(
     phase = cos(temp + phi);
     """,
     "_chirp_phase_hyp_kernel",
-    options=("-std=c++11",),
 )
 
 
@@ -574,11 +575,98 @@ def chirp(t, f0, t1, f1, method="linear", phi=0, vertex_zero=True):
         )
 
 
+def sweep_poly(t, poly, phi=0):
+    """
+    Frequency-swept cosine generator, with a time-dependent frequency.
+
+    This function generates a sinusoidal function whose instantaneous
+    frequency varies with time.  The frequency at time `t` is given by
+    the `poly` array.
+
+    Parameters
+    ----------
+    t : ndarray
+        Times at which to evaluate the waveform.
+    poly : 1-D array_like or instance of numpy.poly1d
+        The desired frequency expressed as a polynomial.  If `poly` is
+        a list or ndarray of length n, then the elements of `poly` are
+        the coefficients of the polynomial, and the instantaneous
+        frequency is
+
+          ``f(t) = poly[0]*t**(n-1) + poly[1]*t**(n-2) + ... + poly[n-1]``
+
+        If `poly` is an instance of cupy.poly1d, then the
+        instantaneous frequency is
+
+          ``f(t) = poly(t)``
+
+    phi : float, optional
+        Phase offset, in degrees, Default: 0.
+
+    Returns
+    -------
+    sweep_poly : ndarray
+        A numpy array containing the signal evaluated at `t` with the
+        requested time-varying frequency.  More precisely, the function
+        returns ``cos(phase + (pi/180)*phi)``, where `phase` is the integral
+        (from 0 to t) of ``2 * pi * f(t)``; ``f(t)`` is defined above.
+
+    See Also
+    --------
+    scipy.signal.sweep_poly
+    chirp
+
+    Notes
+    -----
+
+    If `poly` is an ndarray of length `n`, then the elements of
+    `poly` are the coefficients of the polynomial, and the instantaneous
+    frequency is:
+
+        ``f(t) = poly[0]*t**(n-1) + poly[1]*t**(n-2) + ... + poly[n-1]``
+
+    If `poly` is an instance of `numpy.poly1d`, then the instantaneous
+    frequency is:
+
+          ``f(t) = poly(t)``
+
+    Finally, the output `s` is:
+
+        ``cos(phase + (pi/180)*phi)``
+
+    where `phase` is the integral from 0 to `t` of ``2 * pi * f(t)``,
+    ``f(t)`` as defined above.
+    """
+    # 'phase' is computed in _sweep_poly_phase, to make testing easier.
+    phase = _sweep_poly_phase(t, poly)
+    # Convert to radians.
+    phi *= cupy.pi / 180
+    return cupy.cos(phase + phi)
+
+
+def _sweep_poly_phase(t, poly):
+    """
+    Calculate the phase used by sweep_poly to generate its output.
+
+    See `sweep_poly` for a description of the arguments.
+
+    """
+    if isinstance(poly, cupy.poly1d):
+        poly = poly.coeffs
+
+    # a replacement for `intpoly = np.polyint(poly)`
+    intpoly = cupy.zeros(poly.shape[0] + 1)
+    intpoly[:-1] = poly / cupy.arange(1, poly.shape[0] + 1)[::-1]
+
+    phase = 2 * cupy.pi * cupy.polyval(intpoly, t)
+    return phase
+
+
 UNIT_KERNEL = r'''
 #include <cupy/math_constants.h>
 #include <cupy/carray.cuh>
 #include <cupy/complex.cuh>
-
+#include <cupy/float16.cuh>  // TODO(seberg): Add this via type_headers?
 
 template<typename T>
 __global__ void unit_impulse(const int n, const int iidx, T* out) {
@@ -597,7 +685,7 @@ __global__ void unit_impulse(const int n, const int iidx, T* out) {
 '''
 
 UNIT_MODULE = cupy.RawModule(
-    code=UNIT_KERNEL, options=('-std=c++11',),
+    code=UNIT_KERNEL,
     name_expressions=[f'unit_impulse<{x}>' for x in TYPE_NAMES])
 
 
